#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# implementation of the eckold-sobolev algo
#
# @author Tobias Weber <tweber@ill.fr>
# @date feb-2015, oct-2019
# @license GPLv2
#
# Link to original code for TARKIN: https://doi.org/10.5281/zenodo.4117437
#
# @desc for algorithm: [eck14] G. Eckold and O. Sobolev, NIM A 752, pp. 54-64 (2014)
# @desc for alternate R0 normalisation: [mit84] P. W. Mitchell, R. A. Cowley and S. A. Higgins, Acta Cryst. Sec A, 40(2), 152-160 (1984)
#

# requires numpy version >= 1.10
import numpy as np
import numpy.linalg as la
from MJOLNIR.Geometry import reso


#--------------------------------------------------------------------------
# scattering triangle
# see: https://code.ill.fr/scientific-software/takin/mag-core/blob/master/tools/tascalc/tascalc.py
# 

ksq2E = 2.072124836832


def k2lam(k):
    return 2.*np.pi / k


def get_E(ki, kf):
        return ksq2E * (ki**2. - kf**2.)


def get_scattering_angle(ki, kf, Q):
    c = (ki**2. + kf**2. - Q**2.) / (2.*ki*kf)
    return np.arccos(c)


def get_angle_ki_Q(ki, kf, Q):
    c = (ki**2. + Q**2. - kf**2.) / (2.*ki*Q)
    return np.arccos(c)


def get_angle_kf_Q(ki, kf, Q):
    c = (ki**2. - Q**2. - kf**2.) / (2.*kf*Q)
    return np.arccos(c)


def get_mono_angle(k, d):
    s = np.pi/(d*k)
    angle = np.arcsin(s)
    return angle
#--------------------------------------------------------------------------



#--------------------------------------------------------------------------
# helpers

#
# z rotation matrix
#
def rotation_matrix_3d_z(angle):
    s = np.sin(angle)
    c = np.cos(angle)
    

    return np.array([
        [c, -s, 0],
        [s,  c, 0],
        [0,  0, 1]])


def mirror_matrix(iSize, iComp):
    mat = np.identity(iSize)
    mat[iComp, iComp] = -1.

    return mat;


#
# thin lens equation: 1/f = 1/lenB + 1/lenA
#
def focal_len(lenBefore, lenAfter):
    f_inv = 1./lenBefore + 1./lenAfter
    return 1. / f_inv


#
# optimal mono/ana curvature, 
# see e.g. 
# 	- (Shirane 2002) p. 66
# 	- or nicos/nicos-core.git/tree/nicos/devices/tas/mono.py in nicos
#  - or Monochromator_curved.comp in McStas
#
def foc_curv(lenBefore, lenAfter, tt, bVert):
    f = focal_len(lenBefore, lenAfter)
    s = np.abs(np.sin(0.5*tt))

    if bVert:
        curv = 2. * f*s
    else:
        curv = 2. * f/s

    return curv
#--------------------------------------------------------------------------


#
# mono (and ana) resolution calculation
#
def get_mono_vals(src_w, src_h, mono_w, mono_h,
	dist_src_mono, dist_mono_sample,
	ki, thetam,
	coll_h_pre_mono, coll_h_pre_sample,
	coll_v_pre_mono, coll_v_pre_sample,
	mono_mosaic, mono_mosaic_v,
	inv_mono_curvh, inv_mono_curvv,
	pos_x , pos_y, pos_z,
	refl):

    # A matrix: formula 26 in [eck14]
    A = np.identity(3)

    A_t0 = 1. / mono_mosaic
    A_tx = inv_mono_curvh*dist_mono_sample / np.abs(np.sin(thetam))
    A_t1 = A_t0*A_tx

    A[0,0] = 0.5*reso.sig2fwhm**2. / ki**2. * np.tan(thetam)**2. * \
        ( (2./coll_h_pre_mono)**2. + (2*dist_src_mono/src_w)**2. + A_t0*A_t0 )

    A[0,1] = A[1,0] = 0.5*reso.sig2fwhm**2. / ki**2. * np.tan(thetam) * \
        ( + 2.*(1./coll_h_pre_mono)**2. + 2.*dist_src_mono*(dist_src_mono-dist_mono_sample)/src_w**2. + \
            A_t0**2. - A_t0*A_t1)

    A[1,1] = 0.5*reso.sig2fwhm**2. / ki**2. * \
    ( (1./coll_h_pre_mono)**2. + (1./coll_h_pre_sample)**2. \
        + ((dist_src_mono-dist_mono_sample)/src_w)**2. \
        + (dist_mono_sample/(mono_w*np.abs(np.sin(thetam))))**2. \
        + A_t0*A_t0 - 2.*A_t0*A_t1 + A_t1*A_t1)



    # Av matrix: formula 38 in [eck14]
    # some typos in paper leading to the (false) result of a better Qz resolution when focusing
    # => trying to match terms in Av with corresponding terms in A
    # corresponding pre-mono terms commented out in Av, as they are not considered there
    Av = np.identity(2)

    Av_t0 = 0.5 / (mono_mosaic_v*np.abs(np.sin(thetam)))
    Av_t1 = inv_mono_curvv*dist_mono_sample / mono_mosaic_v

    Av[0,0] = 0.5*reso.sig2fwhm**2. / ki**2. * \
        ( (1./coll_v_pre_sample)**2. + (dist_mono_sample/src_h)**2. + (dist_mono_sample/mono_h)**2. + \
    	Av_t0**2. - 2.*Av_t0*Av_t1 + Av_t1**2. ) 	# typo/missing in paper?

    Av[0,1] = Av[1,0] = 0.5*reso.sig2fwhm**2. / ki**2. * \
        ( dist_src_mono*dist_mono_sample/src_h**2. - Av_t0*Av_t0 + Av_t0*Av_t1 )

    Av[1,1] = 0.5*reso.sig2fwhm**2. / ki**2. * \
        ( (1./(coll_v_pre_mono))**2. + (dist_src_mono/src_h)**2. + Av_t0**2. )



    # B vector: formula 27 in [eck14]
    B = np.array([0,0,0])
    B_t0 = inv_mono_curvh / (mono_mosaic*mono_mosaic*np.abs(np.sin(thetam)))

    B[0] = reso.sig2fwhm**2. * pos_y / ki * np.tan(thetam) * \
        ( 2.*dist_src_mono / src_w**2. + B_t0 )

    B[1] = reso.sig2fwhm**2. * pos_y / ki * \
    ( - dist_mono_sample / (mono_w*np.abs(np.sin(thetam)))**2. + \
        B_t0 - B_t0 * inv_mono_curvh*dist_mono_sample / np.abs(np.sin(thetam)) + \
        (dist_src_mono-dist_mono_sample) / src_w**2. )



    # Bv vector: formula 39 in [eck14]
    Bv = np.array([0,0])

    Bv_t0 = inv_mono_curvv/mono_mosaic_v**2

    # typo in paper?
    Bv[0] = (-1.) *  reso.sig2fwhm**2. * pos_z / ki * \
        ( dist_mono_sample / mono_h**2. + dist_mono_sample / src_h**2. + \
            Bv_t0 * inv_mono_curvv*dist_mono_sample - 0.5*Bv_t0 / np.abs(np.sin(thetam)) )

    # typo in paper?
    Bv[1] = (-1.) * reso.sig2fwhm**2. * pos_z / ki * \
        ( dist_src_mono / (src_h*src_h) + 0.5*Bv_t0/np.abs(np.sin(thetam)) )




    # C scalar: formula 28 in [eck14]
    C = 0.5*reso.sig2fwhm**2. * pos_y**2. * \
	    ( 1./src_w**2. + (1./(mono_w*np.abs(np.sin(thetam))))**2. + \
		    (inv_mono_curvh/(mono_mosaic * np.abs(np.sin(thetam))))**2. )

    # Cv scalar: formula 40 in [eck14] 
    Cv = 0.5*reso.sig2fwhm**2. * pos_z**2. * \
        ( 1./src_h**2. + 1./mono_h**2. + (inv_mono_curvv/mono_mosaic_v)**2. )



    # z components, [eck14], equ. 42
    A[2,2] = Av[0,0] - Av[0,1]*Av[0,1]/Av[1,1]
    B[2] = Bv[0] - Bv[1]*Av[0,1]/Av[1,1]
    D = Cv - 0.25*Bv[1]/Av[1,1]


    # [eck14], equ. 54
    therefl = refl * np.sqrt(np.pi / Av[1,1]) # typo in paper?
    
    return [ A, B, C, D, therefl ]




#
# Eckold algorithm combining the mono and ana resolutions
#
def calc_eck(param):
    twotheta = param["twotheta"] * param["sample_sense"]
    thetam = param["thetam"] * param["mono_sense"]
    thetaa = param["thetaa"] * param["ana_sense"]
    ki_Q = param["angle_ki_Q"] * param["sample_sense"]
    kf_Q = param["angle_kf_Q"] * param["sample_sense"]

    ki = param["ki"]
    kf = param["kf"]
    E = param["E"]
    Q = param["Q"]


    # --------------------------------------------------------------------
    # mono/ana focus
    mono_curvh = param["mono_curvh"]
    mono_curvv = param["mono_curvv"]
    ana_curvh = param["ana_curvh"]
    ana_curvv = param["ana_curvv"]

    if param["mono_is_optimally_curved_h"]:
        mono_curvh = foc_curv(param["dist_src_mono"], param["dist_mono_sample"], np.abs(2.*thetam), False)
    if param["mono_is_optimally_curved_v"]: 
        mono_curvv = foc_curv(param["dist_src_mono"], param["dist_mono_sample"], np.abs(2.*thetam), True)
    if param["ana_is_optimally_curved_h"]: 
        ana_curvh = foc_curv(param["dist_sample_ana"], param["dist_ana_det"], np.abs(2.*thetaa), False)
    if param["ana_is_optimally_curved_v"]: 
        ana_curvv = foc_curv(param["dist_sample_ana"], param["dist_ana_det"], np.abs(2.*thetaa), True)

    inv_mono_curvh = 0.
    inv_mono_curvv = 0.
    inv_ana_curvh = 0.
    inv_ana_curvv = 0.

    if param["mono_is_curved_h"]:
        inv_mono_curvh = 1./mono_curvh
    if param["mono_is_curved_v"]: 
        inv_mono_curvv = 1./mono_curvv
    if param["ana_is_curved_h"]: 
        inv_ana_curvh = 1./ana_curvh
    if param["ana_is_curved_v"]: 
        inv_ana_curvv = 1./ana_curvv
    # --------------------------------------------------------------------


    lam = k2lam(ki)

    coll_h_pre_mono = param["coll_h_pre_mono"]
    coll_v_pre_mono = param["coll_v_pre_mono"]

    if param["use_guide"]:
        coll_h_pre_mono = lam*param["guide_div_h"]
        coll_v_pre_mono = lam*param["guide_div_v"]



    # dict with results
    res = {}

    res["Q_avg"] = np.array([ Q, 0., 0., E ]).flatten()

    # -------------------------------------------------------------------------

    # - if the instruments works in kf=const mode and the scans are counted for
    #   or normalised to monitor counts no ki^3 or kf^3 factor is needed.
    # - if the instrument works in ki=const mode the kf^3 factor is needed.

    # TODO
    #tupScFact = get_scatter_factors(param.flags, param.thetam, param.ki, param.thetaa, param.kf);
    tupScFact = [1., 1., 1.]

    dmono_refl = param["dmono_refl"] * tupScFact[0]
    dana_effic = param["dana_effic"] * tupScFact[1]
    dxsec = tupScFact[2]
    #if param.mono_refl_curve:
    #    dmono_refl *= (*param.mono_refl_curve)(param.ki)
    #if param.ana_effic_curve:
    #    dana_effic *= (*param.ana_effic_curve)(param.kf)


    #--------------------------------------------------------------------------
    # mono part

    [A, B, C, D, dReflM] = get_mono_vals(
        param["src_w"], param["src_h"],
        param["mono_w"], param["mono_h"],
        param["dist_src_mono"], param["dist_mono_sample"],
        ki, thetam,
        coll_h_pre_mono, param["coll_h_pre_sample"],
        coll_v_pre_mono, param["coll_v_pre_sample"],
        param["mono_mosaic"], param["mono_mosaic_v"],
        inv_mono_curvh, inv_mono_curvv,
        param["pos_x"] , param["pos_y"], param["pos_z"],
        dmono_refl)
    #--------------------------------------------------------------------------


    #--------------------------------------------------------------------------
    # ana part
    # equ 43 in [eck14]
    
    ## UPDATED -> Flipped done 19/04/2021
    #
    # y -> z
    # z -> -y
    # w_D <-> h_D
    # w_A <-> h_A
    # alpha_3 <-> alpha_3^v
    # alpha_4 <-> alpha_4^v
    #
    pos_y2 = - param["pos_x"]*np.sin(twotheta) + param["pos_y"]*np.cos(twotheta)

    [E, F, G, H, dReflA] = get_mono_vals(
        param["det_h"], param["det_w"],
        param["ana_h"], param["ana_w"],
        param["dist_ana_det"], param["dist_sample_ana"],
        kf, -thetaa,
        param["coll_v_post_ana"], param["coll_v_post_sample"],
        param["coll_h_post_ana"], param["coll_h_post_sample"],
        param["ana_mosaic_v"], param["ana_mosaic"],
        inv_ana_curvv, inv_ana_curvh,
        param["pos_x"], param["pos_z"], -pos_y2,
        dana_effic)
    #--------------------------------------------------------------------------

    flipper = np.array([[1, 0, 0],[0, 0, -1],[0, 1, 0]])
    
    E = np.dot(flipper,np.dot(E,np.transpose(flipper)))
    F = np.dot(flipper,np.dot(F,np.transpose(flipper)))

    # equ 4 & equ 53 in [eck14]
    dE = (ki**2. - kf**2.) / (2.*Q**2.)
    kipara = Q*(0.5+dE)
    kfpara = Q-kipara
    kperp = np.sqrt(np.abs(kipara**2. - ki**2.))
    kperp *= param["sample_sense"]


    ## UPDATED -> Flipped Not needed 
    #
    # trafo, equ 52 in [eck14]
    T = np.identity(6)
    T[0,3] = T[1,4] = T[2,5] = -1.
    T[3,0] = 2.*ksq2E * kipara
    T[3,3] = 2.*ksq2E * kfpara
    T[3,1] = 2.*ksq2E * kperp
    T[3,4] = -2.*ksq2E * kperp
    T[4,1] = T[5,2] = (0.5 - dE)
    T[4,4] = T[5,5] = (0.5 + dE)

    Tinv = la.inv(T)


    # equ 54 in [eck14]
    Dalph_i = rotation_matrix_3d_z(-ki_Q)
    Dalph_f = rotation_matrix_3d_z(-kf_Q)
    Arot = np.dot(np.dot(np.transpose(Dalph_i), A), Dalph_i)
    Erot = np.dot(np.dot(np.transpose(Dalph_f), E), Dalph_f)

    matAE = np.zeros((6,6))
    matAE[0:3, 0:3] = Arot
    matAE[3:6, 3:6] = Erot

    # U1 matrix
    # typo in paper in quadric trafo in equ 54 (top)?
    U1 = np.dot(np.dot(np.transpose(Tinv), matAE), Tinv)

    # V1 vector
    vecBF = np.zeros(6)
    vecBrot = np.dot(np.transpose(Dalph_i), B)
    vecFrot = np.dot(np.transpose(Dalph_f), F)
    vecBF[0:3] = vecBrot
    vecBF[3:6] = vecFrot
    V1 = np.dot(vecBF, Tinv)



    # --------------------------------------------------------------------------
    # integrate last 2 vars -> equs 57 & 58 in [eck14]

    U2 = reso.quadric_proj(U1, 5);
    U = reso.quadric_proj(U2, 4);

    V2 = reso.quadric_proj_vec(V1, U1, 5);
    V = reso.quadric_proj_vec(V2, U2, 4);

    W = (C + D + G + H) - 0.25*V1[5]/U1[5,5] - 0.25*V2[4]/U2[4,4]

    Z = dReflM*dReflA * np.sqrt(np.pi/np.abs(U1[5,5])) * np.sqrt(np.pi/np.abs(U2[4,4]))
    # --------------------------------------------------------------------------



    # quadratic part of quadric (matrix U)
    # careful: factor -0.5*... missing in U matrix compared to normal gaussian!
    res["reso"] = 2. * U;
    # linear (vector V) and constant (scalar W) part of quadric
    res["reso_v"] = V;
    res["reso_s"] = W;


    if param["sample_sense"] < 0.:
        # mirror Q_perp
        matMirror = mirror_matrix(len(res["reso"]), 1)
        res["reso"] = np.dot(np.dot(np.transpose(matMirror), res["reso"]), matMirror)
        res["reso_v"][1] = -res["reso_v"][1]


    # prefactor and volume
    res["res_vol"] = reso.ellipsoid_volume(res["reso"])

    res["r0"] = Z
    # missing volume prefactor to normalise gaussian,
    # cf. equ. 56 in [eck14] to  equ. 1 in [pop75] and equ. A.57 in [mit84]
    res["r0"] *= res["res_vol"] * np.pi * 3.

    res["r0"] *= np.exp(-W)
    res["r0"] *= dxsec

	# Bragg widths
    res["coherent_fwhms"] = reso.calc_coh_fwhms(res["reso"])
    res["ok"] = True

    if np.isnan(res["r0"]) or np.isinf(res["r0"]) or np.isnan(res["reso"].any()) or np.isinf(res["reso"].any()):
        res["ok"] = False

    return res



#
# test calculation
#
if __name__ == "__main__":
    verbose = True

    cm2A = 1e8
    min2rad = 1./ 60. / 180.*np.pi
    rad2deg = 180. / np.pi

    d_mono = 3.355
    d_ana = 3.355

    ki = 2*np.pi/4
    kf = 2*np.pi/4
    Q = 1.5
    E = get_E(ki, kf)

    sc_senses = [ 1., -1., 1.]

    params = {
        # scattering triangle
        "ki" : ki, "kf" : kf, "E" : E, "Q" : Q,

        # angles
        "twotheta" : get_scattering_angle(ki, kf, Q),
        "thetam" : get_mono_angle(ki, d_mono),
        "thetaa" : get_mono_angle(kf, d_ana),
        "angle_ki_Q" : get_angle_ki_Q(ki, kf, Q),
        "angle_kf_Q" : get_angle_kf_Q(ki, kf, Q),
    
        # scattering senses
        "mono_sense" : sc_senses[0],
        "sample_sense" : sc_senses[1],
        "ana_sense" : sc_senses[2],

        # distances
        "dist_src_mono" : 100. * cm2A,
        "dist_mono_sample" : 100. * cm2A,
        "dist_sample_ana" : 100. * cm2A,
        "dist_ana_det" : 100. * cm2A,

        # component sizes
        "src_w" : 10. * cm2A,
        "src_h" : 10. * cm2A,
        "mono_w" : 10. * cm2A,
        "mono_h" : 10. * cm2A,
        "det_w" : 10. * cm2A,
        "det_h" : 10. * cm2A,
        "ana_w" : 10. * cm2A,
        "ana_h" : 10. * cm2A,

        # focusing
        "mono_curvh" : 0.,
        "mono_curvv" : 0.,
        "ana_curvh" : 0.,
        "ana_curvv" : 0.,
        "mono_is_optimally_curved_h" : False,
        "mono_is_optimally_curved_v" : False,
        "ana_is_optimally_curved_h" : False,
        "ana_is_optimally_curved_v" : False,
        "mono_is_curved_h" : False,
        "mono_is_curved_v" : False,
        "ana_is_curved_h" : False,
        "ana_is_curved_v" : False,

        # collimation
        "coll_h_pre_mono" : 9999. *min2rad,
        "coll_v_pre_mono" : 9999. *min2rad,
        "coll_h_pre_sample" : 9999. *min2rad,
        "coll_v_pre_sample" : 9999. *min2rad,
        "coll_h_post_sample" : 9999. *min2rad,
        "coll_v_post_sample" : 9999. *min2rad,
        "coll_h_post_ana" : 9999. *min2rad,
        "coll_v_post_ana" : 9999. *min2rad,

        # guide
        "use_guide" : True,
        "guide_div_h" : 9999. *min2rad,
        "guide_div_v" : 9999. *min2rad,

        # mosaics
        "mono_mosaic" : 60. *min2rad,
        "mono_mosaic_v" : 60. *min2rad,
        "ana_mosaic" : 60. *min2rad,
        "ana_mosaic_v" : 60. *min2rad,

        # crystal reflectivities
        # TODO, so far always 1
        "dmono_refl" : 1.,
        "dana_effic" : 1.,

        # off-center scattering
        # WARNING: while this is calculated, it is not yet considered in the ellipse plots
        "pos_x" : 0. * cm2A,
        "pos_y" : 0. * cm2A,
        "pos_z" : 0. * cm2A,
    }


    # calculate resolution ellipsoid
    res = calc_eck(params)
    if not res["ok"]:
        print("RESOLUTION CALCULATION FAILED!")
        exit(-1)

    if verbose:
        print("2theta = %g, thetam = %g, thetaa = %g, ki_Q = %g, kf_Q = %g\n" % 
            (params["twotheta"]*rad2deg, params["thetam"]*rad2deg, params["thetaa"]*rad2deg, 
            params["angle_ki_Q"]*rad2deg, params["angle_kf_Q"]*rad2deg))
        print("R0 = %g, Vol = %g" % (res["r0"], res["res_vol"]))
        print("Resolution matrix:\n%s" % res["reso"])
        print("Resolution vector: %s" % res["reso_v"])
        print("Resolution scalar: %g" % res["reso_s"])


    # describe and plot ellipses
    ellipses = reso.calc_ellipses(res["reso"], verbose)
    reso.plot_ellipses(ellipses, verbose)




import os
import sys
import pickle
import re
import MJOLNIR._tools
from MJOLNIR.Data import DataFile
import numpy as np
from PyQt5.QtWidgets import QApplication, QWidget, QFileDialog
from PyQt5.Qt import QApplication
from PyQt5 import Qt

from os.path import expanduser
settingsFile = expanduser("~") # Use home folder for storing settings file

settingsFile = os.path.join(settingsFile,'.MJOLNIRsettings')

allowedRawFilesString = 'Raw ('+' '.join(['*.'+str(x) for x in DataFile.supportedRawFormats])+')'
allowedConvertedFilesString = 'Converted ('+' '.join(['*.'+str(x) for x in DataFile.supportedConvertedFormats])+')'
allowedAllFilesString = 'All Files (*)'

allowedString = ';;'.join([allowedRawFilesString,allowedConvertedFilesString,allowedAllFilesString])
global dataFilesLoaded
# All functions are tested through the test_coonadline.py script but are called through the terminal thus not recorded as tested.
def loadSetting(string): # pragma: no cover
    if Exists():
        with open(settingsFile,"rb") as pickle_in:
            loaded_dict = pickle.load(pickle_in)
        if string in loaded_dict:
            returnValue = loaded_dict[string]
        else:
            returnValue = None
        return returnValue
    else:
        return None

def updateSetting(name,value):# pragma: no cover
    if Exists():
        with open(settingsFile,"rb") as pickle_in:
            loaded_dict = pickle.load(pickle_in)
    else:
        loaded_dict = {}
    loaded_dict[name]=value
    with open(settingsFile,"wb") as pickle_out:
        pickle.dump(loaded_dict, pickle_out)

def Exists(file = settingsFile):# pragma: no cover
    return os.path.isfile(settingsFile)

def extractDataFiles(args,settingsName,oneFile = False):# pragma: no cover
    if not 'DataFile' in args:
        startingPath = loadSetting(settingsName)
        if not startingPath is None:
            if isinstance(startingPath,tuple):
                startingDir = '/'.join([x for x in list(startingPath)[0].split('/')[:-1]])
            else:
                startingDir = startingPath
        else:
            startingDir = None
        
        if 'reuse' in args:
            reuse = args.reuse
        else:
            reuse = False

        if reuse:
            files = startingPath
        else:
            files = loadFiles(startingDir)
        files = tuple(np.unique(files))
        if len(list(files))==0: # No file chosen
            sys.exit()

        directory = os.path.split(files[0])[0]


    else:
        pattern = re.compile(r'^(\d+[-,]?){1,}\d') # Generate pattern to recognize format: Start with digits, and then - or . or nothing repeated at least once, and ending in digit.
        res = pattern.match(args.DataFile[0])
        if not res is None:
            if res.start()==0 and res.end() == len(args.DataFile[0]): # Correct expression is given
                if len(args.DataFile) == 3: # Provided is string, directory and year
                    fileNumbers, directory, year = args.DataFile
                    year = int(year)
                    files = MJOLNIR._tools.fileListGenerator(fileNumbers,directory,year)
                elif len(args.DataFile) == 4:
                    raise NotImplementedError('Using non-CAMEA input is currently not supported... sorry :-/ ')
                else:
                    raise AttributeError('Provided data file arguments does not fit the scheme of being number list, directory, and year! (+ possibly instrument)')
        else:
            files = args.DataFile
    return files

class App(QWidget):
    def __init__(self,startingPath=None):
        super().__init__()
        self.title = 'MJOLNIR Command Line - Load Files'
        self.startingPath = startingPath
        self.initUI()
    
    def initUI(self):
        self.hide()
        self.setWindowTitle(self.title)
        self.openFileNamesDialog()
        
        
        Qt.QCoreApplication.quit()
    
    def openFileNamesDialog(self):
        #options = QFileDialog.Options()
        #options |= QFileDialog.DontUseNativeDialog
        #print(self,"MJOLNIR Command Line - Load Files", allowedString,self.startingPath)
        files, _ = QFileDialog.getOpenFileNames(self,"MJOLNIR Command Line - Load Files", self.startingPath,allowedString)
        global dataFilesLoaded 
        dataFilesLoaded = files
        self.close()
    

    
def loadFiles(startingPath=None):
    app = QApplication(sys.argv)
    ex = App(startingPath=startingPath)
    return dataFilesLoaded


import os
import sys
from typing import Type

sys.path.append('.')
sys.path.append('..')
sys.path.append('../..')

import warnings
import copy
import matplotlib.cm as cm
import matplotlib.gridspec
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.widgets import Slider
from MJOLNIR import _tools
from MJOLNIR._interactiveSettings import Viewer3DSettings, States, cut1DHolder, cancel
import functools

pythonVersion = sys.version_info[0]
pythonSubVersion = sys.version_info[1]

class Viewer3D(object):  
    @_tools.KwargChecker(include=[_tools.MPLKwargs])
    def __init__(self,Data,bins,axis=2, log=False ,ax = None, grid = False, adjustable=True, outputFunction=print, 
                 cmap=None, CurratAxeBraggList=None, plotCurratAxe=False,Ei=None,EfLimits=None, dataset = None, cut1DFunctionRectangle=None,\
                    cut1DFunctionCircle = None, cut1DFunctionRectanglePerp=None,cut1DFunctionRectangleHorizontal=None,cut1DFunctionRectangleVertical=None,
                    cut2DFunctionRectangle=None, cut2DFunctionRectanglePerp=None,cut2DFunctionRectangleHorizontal=None,cut2DFunctionRectangleVertical=None,
                    backgroundSubtraction=False, **kwargs):#pragma: no cover
        """3 dimensional viewing object generating interactive Matplotlib figure. 
        Keeps track of all the different plotting functions and variables in order to allow the user to change between different slicing modes and to scroll through the data in an interactive way.

        Args:

            - Data (3D array): Intensity array in three dimensions. Assumed to have Qx, Qy, and E along the first, second, and third directions respectively.

            - bins (List of 1D arrays): Coordinates of the three directions as returned by the BinData3D functionality of DataSet.

        Kwargs:

            - axis (int): Axis along which the interactive plot slices the data (default 2).

            - log (bool): If true, the log 10 of the intensity is plotted (default False).
            - ax (matplotlib axis): Matplotlib axis into which one plots data (Default None).

            - grid (bool/int): If int, grid will be plotted with zorder=int, if True, grid is plotted at zorder=-10 (Default False).

            - adjustable (bool): If set true, 2 sliders will be present allowing to fine tune the c-axis (Default True)

            - outputFunction (function): Function called on output string (default print)

            - CurratAxeBraggList (list): List of Bragg peaks for which Currat-Axe spurions are to be calculated, [[H1,K1,L1],[H2,K2,L2],..] (default None)

            - plotCurratAxe (bool): Flag to determine whether or not to plot the Currat-Axe spurions
            
            - Ei (list): List of incoming energies (default None)
            
            - EfLimits (float,float): Minimal and maximal Ef for current instrument (default None)

            - dataset (DataSet): Reference to data set used - needed for interactive cutting with (default None)

            - cut1DFunctionRectangle (function): Function to be called when performing an interactive rectangle (default None)

            - cut1DFunctionCircle (function): Function to be called when performing an interactive circle (default None)

            - backgroundSubtraction (bool): Signify if a subtraction has been performed such that interactive cuts work (default False)

        For an example, see the `quick plotting tutorial <../Tutorials/Quick/QuickView3D.html>`_ under scripting tutorials.

        """
        if cmap is None:
            cmap = 'viridis'
        
        self.Ei = Ei
        self.EfLimits = EfLimits

        self.outputFunction = outputFunction


        self.currentData = None
        self.ds = dataset

        self.plotCurratAxe = plotCurratAxe # Set to false but will change to True when correct list is created
        self._CurratAxeBraggList = None
        if len(Data)==4: # If data is provided as I, norm, mon, normcount
            with warnings.catch_warnings() as w:
                warnings.simplefilter("ignore")
                self.Data = np.divide(Data[0]*Data[3],Data[1]*Data[2])
            
            self.Counts,self.Monitor,self.Normalization,self.NormCounts = Data
            self.allData = True
        elif len(Data) == 2: # From bin3D with no normalization or monitor
            with warnings.catch_warnings() as w:
                warnings.simplefilter("ignore")
                self.Data = np.divide(Data[0],Data[1])
            self.allData = False
        else:
            self.Data = Data
            self.allData = False
        if log:
            self.Data = np.log10(self.Data+1e-20)
        self.bins = bins
        self.dataLimits = [np.nanmin(Data),np.nanmax(Data)]

        gs = matplotlib.gridspec.GridSpec(1, 2, width_ratios=[4, 1]) 
        
        if not grid == False:
            gridArg = grid
            if isinstance(gridArg,(bool)):
                self.grid = True
                self.gridZOrder=-10
            elif isinstance(gridArg,(int,float)):
                self.grid = True
                self.gridZOrder=gridArg
        else:
            self.grid = False
            self.gridZOrder = 0

        if ax is None: # TODO: REDO with actual correct axes!
            self.figure = plt.figure()
            self.ax = plt.subplot(gs[0])#self.figure.add_subplot(111)
            self.ax.isActive = lambda : True
            self.ax.drawState = States.INACTIVE
            self.xlabel = r'Qx [$A^{-1}$]'
            self.ylabel = r'Qy [$A^{-1}$]'
            self.zlabel = r'E [meV]'
            self.rlu = False
            self._axes = [self.ax]
        else:
            if isinstance(ax,plt.Axes): # Assuming only RLU - energy plot is provided
                self.axRLU = ax
                self.figure = ax.get_figure() # Get the correct figure
                self.axNorm,ax2  = self.figure.subplots(1,2,gridspec_kw={'width_ratios':[4, 1]}) # Create figure on top of the other
                ax2.remove() # Remove the excess figure
                
                self.axRLU.set_position(self.axNorm.get_position()) # Update RLU to correct position

                self._axes = [self.axNorm,self.axNorm,self.axRLU]
                self._axes[0].set_xlabel(r'Qx [$A^{-1}$]')
                self._axes[0].set_ylabel(r'E [meV]')
                self._axes[1].set_xlabel(r'Qy [$A^{-1}$]')
                self._axes[1].set_ylabel(r'E [meV]')
                self.ax = self.axNorm
                self.xlabel = r'Qx [$A^{-1}$]'
                self.ylabel = r'Qy [$A^{-1}$]'
                self.zlabel = 'E [meV]'
                self.rlu = True
            elif len(ax)==3: # All axes provided in order QxE,QyE,QxQy
                self.axQxE = ax[0]
                self.axQyE = ax[1]
                self.axRLU = ax[2]
                self.figure = self.axQyE.get_figure() # Get the correct figure
                self.axNorm,ax2  = self.figure.subplots(1,2,gridspec_kw={'width_ratios':[4, 1]}) # Create figure on top of the other
                ax2.remove() # Remove the excess figure
                
                self.axQxE.set_position(self.axNorm.get_position()) # Update axQxE to correct position
                self.axQyE.set_position(self.axNorm.get_position()) # Update axQyE to correct position
                self.axRLU.set_position(self.axNorm.get_position()) # Update axRLU to correct position
                self._axes = [self.axQxE,self.axQyE,self.axRLU]
                
                self.ax = self.axNorm
                hkl = ['H','K','L']
                xlabelSplit = self.axQyE.get_xlabel().replace(' [RLU]','').split(',')
                ylabelSplit = self.axQxE.get_xlabel().replace(' [RLU]','').split(',')
                self.xlabel = '\n'.join(['{}: '.format(hkl[i])+'{:+.3f}'.format(float(xlabelSplit[i])) for i in range(len(xlabelSplit))])
                self.ylabel = '\n'.join(['{}: '.format(hkl[i])+'{:+.3f}'.format(float(ylabelSplit[i])) for i in range(len(ylabelSplit))])
                self.zlabel = 'E [meV]'
                self.rlu = True
                
                self.EnergySliderTransform=[self.axQxE._length,self.axQyE._length,1.0] # Factor to divide the Energy slider value with (only applicable for QE axes)

            else:
                raise AttributeError('Number of provided axes is {} but only 1 or 3 is accepted.'.format(len(ax)))


        dQx = np.diff(self.bins[0][:2,0,0])[0]
        dQy = np.diff(self.bins[1][0,:2,0])[0]
        ddE = np.diff(self.bins[2][0,0,:2])[0]
        self.dQE = [dQx,dQy,ddE]

        for ax in self._axes:
            ax.backgroundSubtraction = backgroundSubtraction
            ax.dQE = self.dQE
            ax.get_clim = self.get_clim
        self.figure.set_size_inches(11,7)
        self.value = 0
        self.figure.subplots_adjust(bottom=0.25)
        self.cmap = cmap # Update to accommodate deprecation warning
        self.value = 0
        self.setAxis(2)
        # Set up interactive generation of 1DCuts


        viewAxis = axis
        axis_color='white'
        
        self.figure.canvas.mpl_connect('key_press_event',lambda event: onkeypress(event, self) )
        self.figure.canvas.mpl_connect('scroll_event',lambda event: onscroll(event, self))
        
        zeroPoint = np.argmin(np.abs(0.5*(self.Z[0,0][1:]+self.Z[0,0][:-1])))
        
    
        self.Energy_slider_ax = self.figure.add_axes([0.15, 0.1, 0.65, 0.03])
        

        self.Energy_slider = Slider(self.Energy_slider_ax, label=self.label, valmin=self.lowerLim, valmax=self.upperLim, valinit=zeroPoint)
        self.Energy_slider.valtext.set_visible(False)
        
        self.Energy_slider.on_changed(lambda val: sliders_on_changed(self,val))
            
        if self.rlu:
            self.units = ['','',' meV']
        else:
            self.units = [' 1/AA',' 1/AA',' meV']
            
        textposition = [self.Energy_slider_ax.get_position().p1[0]+0.005,self.Energy_slider_ax.get_position().p0[1]+0.005]
        self.text = self.figure.text(textposition[0], textposition[1],s=self.stringValue())
        self.shading = 'flat'
        #self.imcbaxes = self.figure.add_axes([0.0, 0.2, 0.2, 0.7])
        #self.im = self.ax.imshow(self.masked_array[:,:,self.value].T,cmap=self.cmap,extent=[self.X[0],self.X[-1],self.Y[0],self.Y[-1]],origin='lower')
        if self.shading=='flat':
            self.ax.grid(False)
            self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=cmap)
        elif self.shading=='gouraud':  # pragma: no cover
            XX = 0.5*(self.X[:-1,:-1,self.value]+self.X[1:,1:,self.value]).T
            YY = 0.5*(self.Y[:-1,:-1,self.value]+self.Y[1:,1:,self.value]).T
            self.ax.grid(False)
            self.im = self.ax.pcolormesh(XX,YY,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=cmap) # ,vmin=1e-6,vmax=6e-6
        else:
            raise AttributeError('Did not understand shading {}.'.format(self.shading))
        if self.grid:
            self.ax.grid(self.grid,zorder=self.gridZOrder)
        else:
            self.ax.grid(self.grid)

        self._caxis = self.im.get_clim()
        self.figpos = [0.125,0.25,0.63,0.63]#self.ax.get_position()
        
        self.cbaxes = self.figure.add_axes([0.8, 0.2, 0.03, 0.7])
        self.colorbar = self.figure.colorbar(self.im,cax = self.cbaxes)
        warnings.simplefilter("ignore")
        #self.figure.tight_layout(rect=[0,0.1,0.9,0.9])
        warnings.simplefilter("once")

        self.text.set_text(self.stringValue())
        xlim = self.ax.get_xlim()
        ylim = self.ax.get_ylim()
        #if self.axis == 2:
        #    self.ax.set_xlim(np.min([xlim[0],ylim[0]]),np.max([xlim[1],ylim[1]]))
        

        self.Energy_slider.set_val(self.value)
        if self.rlu:

            self.axRLU.onClick = lambda event: eventdecorator(onclick,self,self.axRLU,event,outputFunction=outputFunction)#,extra=' axRLU')
            self.axRLU._button_press_event = self.figure.canvas.mpl_connect('button_press_event', self.axRLU.onClick)

            def updateLimitsAxRLU(self,lower,upper):
                self.EMin = lower
                self.EMax = upper
            self.axRLU.updateLimits = lambda lower,upper: updateLimitsAxRLU(self.axRLU,lower,upper)
            
            self.axQxE.onClick = lambda event: eventdecorator(onclick,self,self.axQxE,event,outputFunction=outputFunction)#,extra=' axQxE')
            self.axQxE._button_press_event = self.figure.canvas.mpl_connect('button_press_event', self.axQxE.onClick)

            def updateLimitsAxQ(self,lower,upper):
                self.QPoints = [self.sample.calculateHKLToQxQy(*x) for x in np.array([self.calculateRLU(-1,0)[0],self.calculateRLU(1,0)[0]])]
                
            self.axQxE.updateLimits = lambda lower,upper: updateLimitsAxQ(self.axQxE,lower,upper)
            self.axQyE.updateLimits = lambda lower,upper: updateLimitsAxQ(self.axQyE,lower,upper)
            self.axQyE.onClick = lambda event: eventdecorator(onclick,self,self.axQyE,event,outputFunction=outputFunction)#,extra=' axQyE')
            self.axQyE._button_press_event = self.figure.canvas.mpl_connect('button_press_event', self.axQyE.onClick)

            self.axRLU.rlu =  self.axQxE.rlu = self.axQyE.rlu = True
            
            # Set up initial active axes
            self.axRLU.isActive = True
            self.axQxE.isActive = False
            self.axQyE.isActive = False

            if not cut1DFunctionRectangle is None:
                self.axRLU.cut1DFunctionRectangle = lambda dr: cut1DFunctionRectangle(viewer=self,dr=dr)

            if not cut1DFunctionCircle is None:
                self.axRLU.cut1DFunctionCircle = lambda dr: cut1DFunctionCircle(viewer=self,dr=dr)

            
            if not cut1DFunctionRectanglePerp is None:
                self.axQxE.cut1DFunctionRectanglePerpendicular = lambda dr: cut1DFunctionRectanglePerp(viewer=self,dr=dr)
                self.axQyE.cut1DFunctionRectanglePerpendicular = lambda dr: cut1DFunctionRectanglePerp(viewer=self,dr=dr)

            if not cut1DFunctionRectangleHorizontal is None:
                self.axQxE.cut1DFunctionRectangleHorizontal = lambda dr: cut1DFunctionRectangleHorizontal(viewer=self,dr=dr)
                self.axQyE.cut1DFunctionRectangleHorizontal = lambda dr: cut1DFunctionRectangleHorizontal(viewer=self,dr=dr)

            if not cut1DFunctionRectangleVertical is None:
                self.axQxE.cut1DFunctionRectangleVertical = lambda dr: cut1DFunctionRectangleVertical(viewer=self,dr=dr)
                self.axQyE.cut1DFunctionRectangleVertical = lambda dr: cut1DFunctionRectangleVertical(viewer=self,dr=dr)

            
            if not cut2DFunctionRectangle is None:
                self.axRLU.cut2DFunctionRectangle = lambda dr: cut2DFunctionRectangle(viewer=self,dr=dr)

            if not cut1DFunctionCircle is None:
                self.axRLU.cut1DFunctionCircle = lambda dr: cut1DFunctionCircle(viewer=self,dr=dr)

            
            if not cut2DFunctionRectanglePerp is None:
                self.axQxE.cut2DFunctionRectanglePerpendicular = lambda dr: cut2DFunctionRectanglePerp(viewer=self,dr=dr)
                self.axQyE.cut2DFunctionRectanglePerpendicular = lambda dr: cut2DFunctionRectanglePerp(viewer=self,dr=dr)

            if not cut2DFunctionRectangleHorizontal is None:
                self.axQxE.cut2DFunctionRectangleHorizontal = lambda dr: cut2DFunctionRectangleHorizontal(viewer=self,dr=dr)
                self.axQyE.cut2DFunctionRectangleHorizontal = lambda dr: cut2DFunctionRectangleHorizontal(viewer=self,dr=dr)

            if not cut2DFunctionRectangleVertical is None:
                self.axQxE.cut2DFunctionRectangleVertical = lambda dr: cut2DFunctionRectangleVertical(viewer=self,dr=dr)
                self.axQyE.cut2DFunctionRectangleVertical = lambda dr: cut2DFunctionRectangleVertical(viewer=self,dr=dr)

        else:
            self.ax._button_press_event = self.figure.canvas.mpl_connect('key_press_event',lambda event: onkeypress(event, self) )
            self.onClick = lambda event: eventdecorator(onclick,self,self.ax,event,outputFunction=outputFunction)
            self.button_press_event = self.figure.canvas.mpl_connect('button_press_event', self.onClick)

            self.ax.rlu =  False
            
        
        
        try:
            maxVal = np.nanmax(self.masked_array[np.isfinite(self.masked_array)])
        except ValueError:
            maxVal = 1
        try:
            minVal = np.nanmin(self.masked_array[np.isfinite(self.masked_array)])
        except ValueError:
            minVal = 0
        self.caxis = [minVal,maxVal]
        if self.grid:
            self.ax.grid(self.grid,zorder=self.gridZOrder)
        else:
            self.ax.grid(self.grid)

        self.setAxis(viewAxis) # Set view plane to correct
        ## Hack for this to look nice as just changing direction does not render correctly
        self.setPlane(1)
        self.setPlane(0)

        if not self.EfLimits is None:
            self.Ef = np.arange(*self.EfLimits,self.dQE[2])
            
            self.CurratAxeBraggList = CurratAxeBraggList

        if adjustable and pythonVersion>2 and pythonSubVersion>5:

            ax_cmin = plt.axes([0.87, 0.1, 0.05, 0.7])
            ax_cmax  = plt.axes([0.93, 0.1, 0.05, 0.7])

            addColorbarSliders(self,c_min=self.caxis[0],c_max=self.caxis[1],c_minval=self.caxis[0],\
                c_maxval=self.caxis[1],ax_cmin=ax_cmin,ax_cmax=ax_cmax,log=False)

    @property 
    def caxis(self):
        return self._caxis

    @caxis.getter
    def caxis(self):
        return self._caxis

    @caxis.setter
    def caxis(self,caxis):
        ErrMsg = 'Provided caxis is not of correct format. Expected 2 values but recieved "{}" of type {}'
        if not isinstance(caxis,(list,np.ndarray,tuple)):
            raise AttributeError(ErrMsg.format(caxis,type(caxis)))
        if len(list(caxis))!=2:
            raise AttributeError(ErrMsg.format(caxis,type(caxis)))
        self._caxis = caxis
        self.im.set_clim(caxis)
        cmin,cmax = caxis
        fig = self.ax.get_figure()
        #if not _internal:
        if hasattr(fig,'s_cmin'): # The method addColorbarSliders has been called
            for s in [fig.s_cmin,fig.s_cmax]:
                if s._log:
                    s.valmin = np.log10(cmin)
                    s.valmax = np.log10(cmax)
                else:
                    s.valmin = cmin
                    s.valmax = cmax
                    s.ax.set_ylim(s.valmin,s.valmax)
            fig.s_cmin.set_val(cmin)
            fig.s_cmax.set_val(cmax)

        self.colorbar.update_normal(self.im)

    def set_clim(self,cmin,cmax=None):
        if cmax is None:
            self.caxis = cmin
        else:
            self.caxis = (cmin,cmax)

    def get_clim(self):
        return self.caxis

    def setAxis(self,axis):
        if hasattr(self,'im'): # this function is also called before any plot has been performed
            self.im.set_array(self.emptyData) # Set data in current im to a fully masked data set
        if axis==2:
            if self.rlu:
                if hasattr(self.ax,'_button_press_event'):
                    self.ax.get_figure().canvas.mpl_disconnect(self.ax._button_press_event)
                self.ax.isActive = False
                self.figure.delaxes(self.ax)
                self.ax = self.figure.add_axes(self._axes[axis])
                self.ax.isActive = True
                if hasattr(self.ax,'onClick'):
                    self.ax._button_press_event = self.ax.get_figure().canvas.mpl_connect('button_press_event', self.ax.onClick)
            else:
                self.ax.set_xlabel(self.xlabel)
                self.ax.set_ylabel(self.ylabel)
            axes = (0,1,2)
            label = self.zlabel#self.ax.get_ylabel
        elif axis==1:  # pragma: no cover
            if self.rlu:
                if hasattr(self.ax,'_button_press_event'):
                    self.ax.get_figure().canvas.mpl_disconnect(self.ax._button_press_event)
                self.ax.isActive = False
                self.figure.delaxes(self.ax)
                self.ax = self.figure.add_axes(self._axes[axis])
                self.ax.isActive = True
                if hasattr(self.ax,'onClick'):
                    self.ax._button_press_event = self.ax.get_figure().canvas.mpl_connect('button_press_event', self.ax.onClick)
            else:
                self.ax.set_xlabel(self.xlabel)
                self.ax.set_ylabel(self.zlabel)
            axes = (0,2,1)
            label =  self.ylabel#self.ax.get_ylabel
        elif axis==0:  # pragma: no cover
            if self.rlu:
                if hasattr(self.ax,'_button_press_event'):
                    self.ax.get_figure().canvas.mpl_disconnect(self.ax._button_press_event)
                self.ax.isActive = False
                self.figure.delaxes(self.ax)
                self.ax = self.figure.add_axes(self._axes[axis])
                self.ax.isActive = True
                if hasattr(self.ax,'onClick'):
                    self.ax._button_press_event = self.ax.get_figure().canvas.mpl_connect('button_press_event', self.ax.onClick)
            else:
                self.ax.set_xlabel(self.ylabel)
                self.ax.set_ylabel(self.zlabel)
            axes = (1,2,0)
            label =  self.xlabel#self.ax.get_xlabel()
            
        else:
            raise AttributeError('Axis provided not recognized. Should be 0, 1, or 2 but got {}'.format(axis))
        #self.ax.format_coord = self._axes[axis].format_coord
        if hasattr(self.ax,'_step'):
            self.ax._step=self.calculateValue()
        X=self.bins[axes[0]].transpose(axes)
        Y=self.bins[axes[1]].transpose(axes)
        Z=self.bins[axes[2]].transpose(axes)

        
        masked_array = np.ma.array(self.Data, mask=np.isnan(self.Data)).transpose(axes)
        self.emptyData = masked_array[:,:,0].T.flatten().copy()
        self.emptyData.mask = np.ones_like(self.emptyData,dtype=bool)
        self._axesChanged = True
        upperLim = self.Data.shape[axis]-1
        self.label = label
        self.X = X
        self.Y = Y
        self.Z = Z
        self.masked_array = masked_array
        self.axes = axes
        self.upperLim = upperLim
        self.lowerLim = 0
        self.axis = axis

    def calculateValue(self,index=None):
        if index is None:
            index = int(self.value)
        try:
            val = 0.5*(self.Z[0,0,index+1]+self.Z[0,0,index])
        except:
            val = 0.5*(2*self.Z[0,0,index]-self.Z[0,0,index-1])
        if hasattr(self,'EnergySliderTransform'):
            val/=self.EnergySliderTransform[self.axis]
        return val
        
    def stringValue(self):
        unit = self.units[self.axis]
        val = self.calculateValue()
        return str(np.round(val,2))+unit
    
    def setProjection(self,value):
        """Change projection between Qx,Qy, and E, or along principal, orthogonal Q direction, or E if plotting in RLU."""
        self.figure.canvas.key_press_event(str(value))

    def setPlane(self,value):
        """Change plotting plane to new along same axis"""
        self.Energy_slider.set_val(value)
        
    @property
    def CurratAxeBraggList(self):
        return self._CurratAxeBraggList
    
    @CurratAxeBraggList.setter
    def CurratAxeBraggList(self,peaks):
        self._CurratAxeBraggList = peaks

        if peaks is None:
            self.plotCurratAxe = False
        elif np.array(self.plotCurratAxe).size ==0:
            self.plotCurratAxe = False
        elif self.rlu :
            self.monoPoints = self.axRLU.sample.CurratAxe(Ei=self.Ei,Ef=self.Ef,Bragg=self.CurratAxeBraggList,spurionType='Monochromator',HKL=False)[:,:,:,:2]
            self.anaPoints = self.axRLU.sample.CurratAxe(Ei=self.Ei,Ef=self.Ef,Bragg=self.CurratAxeBraggList,spurionType='Analyser',HKL=False)[:,:,:,:2]

            self.dE = np.repeat((self.Ei[:,np.newaxis]-self.Ef[np.newaxis])[np.newaxis],len(self.CurratAxeBraggList),axis=0)
            dEIndex = np.round((self.dE-self.bins[2][0,0,0])/self.dQE[2])
            dQxIndexMono = np.round((self.monoPoints[:,:,:,0]-self.bins[0][0,0,0])/self.dQE[0])
            dQyIndexMono = np.round((self.monoPoints[:,:,:,1]-self.bins[1][0,0,0])/self.dQE[1])
            dQxIndexAna = np.round((self.anaPoints[:,:,:,0]-self.bins[0][0,0,0])/self.dQE[0])
            dQyIndexAna = np.round((self.anaPoints[:,:,:,1]-self.bins[1][0,0,0])/self.dQE[1])

            self.CurratAxeIndicesMono = np.array([dQxIndexMono,dQyIndexMono,dEIndex])
            self.CurratAxeIndicesAna = np.array([dQxIndexAna,dQyIndexAna,dEIndex])
            try:
                self.plot()
                self.plotCurratAxe = True
            except:
                pass
            
        else:
            self.plotCurratAxe = False

    @CurratAxeBraggList.getter
    def CurratAxeBraggList(self):
        return self._CurratAxeBraggList

    @property
    def plotCurratAxe(self):
        return self._plotCurratAxe
    
    @plotCurratAxe.setter
    def plotCurratAxe(self,value):
        self._plotCurratAxe = value
        if hasattr(self,'_axes'):
            # for ax in self._axes:
            #     for plotter in ['BraggScatterMono','BraggScatterAna']:
            #         if hasattr(ax,plotter):
            #             try:
            #                 getattr(ax,plotter).remove()
            #             except ValueError:
            #                 pass
            if self._plotCurratAxe:
                self.plot()
    
    def plot(self):

        self.text.set_text(self.stringValue())
        try:
            #self.im.set_array(self.emptyData)
            pass
        except TypeError:
            pass
        if self._axesChanged:
            tempData = np.ma.array(self.im.get_array().T)
            tempData.mask = np.ones_like(tempData,dtype=bool)
            if tempData.shape[0] != self.X.shape[0]:
                tempData = tempData.T
            self.im.set_array(tempData)
            self._axesChanged = False
        else:
            self.im.set_array(self.masked_array[:,:,int(self.value)].T.flatten())
        
        if not self.CurratAxeBraggList is None and not self.Ei is None and self.plotCurratAxe is True and self.rlu is True:

            insidePointsMono = self.monoPoints[np.isclose(self.CurratAxeIndicesMono[self.axis],self.Energy_slider.val)]
            insidePointsAna = self.anaPoints[np.isclose(self.CurratAxeIndicesAna[self.axis],self.Energy_slider.val)]
            if not hasattr(self.ax,'BraggScatterMono'):
                self.ax.BraggScatterMono = self.ax.plot([],[],zorder=20, linestyle="", marker="o", mfc='none', color='r',label='Currat-Axe Monochromator')[0]
            if not hasattr(self.ax,'BraggScatterAna'):
                self.ax.BraggScatterAna = self.ax.plot([],[],zorder=20, linestyle="", marker="o", mfc='none', color='k',label='Currat-Axe Analyzer')[0]
            
            
            self.ax.BraggScatterMono.set_data([],[])
            self.ax.BraggScatterAna.set_data([],[])
            
            if len(insidePointsMono) > 0: # Only if there are any points to plot, do it
                insidePointsMono = insidePointsMono.T
                
                                
                
                if self.axis == 2:
                    insidePointsAna = insidePointsAna.T
                    self.ax.BraggScatterMono.set_data(*insidePointsMono)
                    self.ax.BraggScatterAna.set_data(*insidePointsAna)
                else:
                    energy = self.dE[np.isclose(self.CurratAxeIndicesMono[self.axis],self.Energy_slider.val)]
                    
                    plotPointsMoni = np.array([insidePointsMono[1-self.axis],energy])
                    self.ax.BraggScatterMono.set_data(*plotPointsMoni)
                    
            if len(insidePointsAna) > 0 and self.axis != 2: # Only if there are any points to plot, do it
                insidePointsAna = insidePointsAna.T
                
                energy = self.dE[np.isclose(self.CurratAxeIndicesAna[self.axis],self.Energy_slider.val)]

                plotPointsAna = np.array([insidePointsAna[1-self.axis],energy])
                self.ax.BraggScatterAna.set_data(*plotPointsAna)
            #self.ax.

        self.im.set_clim(self.caxis)
        self.ax.set_position(self.figpos)
        xlim = self.ax.get_xlim()
        ylim = self.ax.get_ylim()
        if self.axis == 2:
            pass
        if self.grid:
            self.ax.grid(self.grid,zorder=self.gridZOrder)
        else:
            self.ax.grid(self.grid)
    def set_title(self,title):
        self.ax.set_title(title)

    def saveToFile(self,folder,extension,gui=None):
        """Save all planes to files
        
        Args:

            - folder (path): Folder in which to save files

            - extension (str): name of wanted extension. Must be accepted by matplotlib

        Kwargs:

            - gui (PyQt5 MainApplication): Used for MJOLNIR Gui
        """
        XLength,YLength,ZLength = self.Data.shape

        startProjection = self.axis
        startPosition = self.Energy_slider.val

        totalFigures = XLength+YLength+ZLength
        if not gui is None:
            gui.setProgressBarMaximum(totalFigures)
            progress = 0
            gui.setProgressBarValue(progress) 
        self.setProjection(2)
        for zidx in range(ZLength):
            self.setPlane(zidx)
            Energy = self.bins[2][0,0,zidx]
            self.figure.savefig(os.path.join(folder,'E{}.{}'.format('{:.2f}'.format(Energy).replace('.','p').replace('-','m'),extension)))
            if not gui is None:
                progress += 1
                gui.setProgressBarValue(progress)

        self.setProjection(0)

        for xidx in range(XLength):
            self.setPlane(xidx)
            position = self.bins[0][xidx,0,0]
            self.figure.savefig(os.path.join(folder,'X{}.{}'.format('{:.2f}'.format(position).replace('.','p').replace('-','m'),extension)))
            if not gui is None:
                progress += 1
                gui.setProgressBarValue(progress)
            
        self.setProjection(1)

        for yidx in range(YLength):
            self.setPlane(yidx)
            position = self.bins[1][0,yidx,0]
            self.figure.savefig(os.path.join(folder,'Y{}.{}'.format('{:.2f}'.format(position).replace('.','p').replace('-','m'),extension)))
            if not gui is None:
                progress += 1
                gui.setProgressBarValue(progress)
            
        self.setProjection(startProjection)
        self.setPlane(startPosition)


def eventdecorator(function,self,ax,event,*args,**kwargs):# pragma: no cover
    if event.xdata is not None and ax.in_axes(event) and ax.isActive:
        try:
            C = ax.get_figure().canvas.cursor().shape() # Only works for pyQt5 backend
        except:
            pass
        else:
            if C != 0 or ax.drawState != States.INACTIVE:
                return
        return function(self,event.xdata,event.ydata,*args,**kwargs)


def onclick(self,x,y,returnText=False, outputFunction=print,extra=None): # pragma: no cover
    idz = self.value
    axis = self.axis

    XX,YY = self.X[:,:,idz],self.Y[:,:,idz]
    XX = 0.25*(XX[:-1,:-1]+XX[1:,:-1]+XX[:-1,1:]+XX[1:,1:])
    YY = 0.25*(YY[:-1,:-1]+YY[1:,:-1]+YY[:-1,1:]+YY[1:,1:])
    idx = np.unravel_index(np.argmin(np.abs(XX-x)),XX.shape)[0]
    idy = np.unravel_index(np.argmin(np.abs(YY-y)),YY.shape)[1]
    I = self.masked_array[idx,idy,idz]

    masked = np.ma.is_masked(I)
    printString = ''
    printString+=self.ax.format_coord(x, y)+', '

    if masked:
        I = np.nan

    printString+='I = {:.4E}'.format(I)
    
    if self.allData is True and not masked:
        if self.axis == 0:
            flipper = [2,0,1]
        elif self.axis == 1:
            flipper = [0,2,1]
        else:
            flipper = [0,1,2]
        ID = np.array([idx,idy,idz])[flipper]
        cts = self.Counts[ID[0],ID[1],ID[2]]
        Norm = self.Normalization[ID[0],ID[1],ID[2]]
        Mon = self.Monitor[ID[0],ID[1],ID[2]]
        NC = self.NormCounts[ID[0],ID[1],ID[2]]
        printString+=', Cts = {:}, Norm = {:.3f}, Mon = {:d}, NormCount = {:d}'.format(cts,Norm,int(Mon),int(NC))
    if not extra is None:
        printString+=extra
    if returnText:
        return printString
    else:
        if hasattr(self.ax,'suppressPrint'):
            if not self.ax.suppressPrint:
                outputFunction(printString)
        else:
            outputFunction(printString)
        
def onkeypress(event,self): # pragma: no cover
    if event.key in Viewer3DSettings['upwards']:
        increaseAxis(event,self)
    elif event.key in Viewer3DSettings['downwards']:
        decreaseAxis(event,self)
    elif event.key in Viewer3DSettings['home']:
        self.Energy_slider.set_val(self.Energy_slider.valmin)
    elif event.key in Viewer3DSettings['end']:
        self.Energy_slider.set_val(self.Energy_slider.valmax)

    elif event.key in Viewer3DSettings['QxE']:
        if self.axis!=0:
            reloadslider(self,0)
            del self.im
            #self.currentData = None
            self.ax.grid(False)
            if self.shading=='flat':
                self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
            elif self.shading=='gouraud':
                self.im = self.ax.pcolormesh(0.5*(self.X[:-1,:-1,0]+self.X[1:,1:,0]).T,0.5*(self.Y[:-1,:-1,0]+self.Y[1:,1:,0]).T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
            else:
                raise AttributeError('Did not understand shading {}.'.format(self.shading))
            if self.grid:
                self.ax.grid(self.grid,zorder=self.gridZOrder)
            else:
                self.ax.grid(self.grid)
            self.im.set_clim(self.caxis)
            self.Energy_slider.set_val(0)
            self.plot()
            self.ax.set_xlim([np.min(self.X),np.max(self.X)])
            self.ax.set_ylim([np.min(self.Y),np.max(self.Y)])
    elif event.key in Viewer3DSettings['QyE']:
        if self.axis!=1:
            reloadslider(self,1)
            del self.im
            #self.currentData = None
            self.ax.grid(False)
            if self.shading=='flat':
                self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
            elif self.shading=='gouraud':
                self.im = self.ax.pcolormesh(0.5*(self.X[:-1,:-1]+self.X[1:,:1:]).T,0.5*(self.Y[:-1,-1]+self.Y[1:,1:]).T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
            else:
                raise AttributeError('Did not understand shading {}.'.format(self.shading))
            if self.grid:
                self.ax.grid(self.grid,zorder=self.gridZOrder)
            else:
                self.ax.grid(self.grid)
            self.im.set_clim(self.caxis)
            self.Energy_slider.set_val(0)
            self.plot()
            self.ax.set_xlim([np.min(self.X),np.max(self.X)])
            self.ax.set_ylim([np.min(self.Y),np.max(self.Y)])
    elif event.key in Viewer3DSettings['QxQy']:
        if self.axis!=2:
            reloadslider(self,2)
            del self.im
            #self.currentData = None
            self.ax.grid(False)
            if self.shading=='flat':
                self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
            elif self.shading=='gouraud':
                XX = 0.5*(self.X[:-1,:-1,self.value]+self.X[1:,1:,self.value]).T
                YY = 0.5*(self.Y[:-1,:-1,self.value]+self.Y[1:,1:,self.value]).T
                self.im = self.ax.pcolormesh(XX,YY,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
            else:
                raise AttributeError('Did not understand shading {}.'.format(self.shading))
            if self.grid:
                self.ax.grid(self.grid,zorder=self.gridZOrder)
            else:
                self.ax.grid(self.grid)
            self.im.set_clim(self.caxis)
            self.Energy_slider.set_val(0)
            self.plot()
            self.ax.set_xlim([np.min(self.X),np.max(self.X)])
            self.ax.set_ylim([np.min(self.Y),np.max(self.Y)])


def reloadslider(self,axis):
    try:
        cancel(self.ax,axis)
    except:
        pass#print('Nope')
    self.setAxis(axis)
    self.Energy_slider.set_val(0)
    self.Energy_slider.label.remove()
    self.Energy_slider.disconnect_events()#(self.Energy_slider.cids[0])
    self.Energy_slider.vline.set_visible(False)
    
    del self.Energy_slider
    
    zeroPoint = np.argmin(np.abs(0.5*(self.Z[0,0][1:]+self.Z[0,0][:-1])))
    self.Energy_slider = Slider(self.Energy_slider_ax, label=self.label, valmin=self.lowerLim, valmax=self.upperLim, valinit=zeroPoint)
    self.Energy_slider.valtext.set_visible(False)
    self.Energy_slider.on_changed(lambda val: sliders_on_changed(self,val))
    self.value=0
    self.im.remove()
    
        
def onscroll(event,self): # pragma: no cover
    if(event.button in Viewer3DSettings['upwardsScroll']):
        increaseAxis(event,self)
    elif event.button in Viewer3DSettings['downwardsScroll']:
        decreaseAxis(event,self)


def increaseAxis(event,self): # pragma: no cover
    self.Energy_slider.set_val(self.Energy_slider.val+1)
    
    
def decreaseAxis(event,self): # pragma: no cover
    self.Energy_slider.set_val(self.Energy_slider.val-1)   
    

def sliders_on_changed(self,val): # pragma: no cover
    value = int(np.round(val))
    
    if value>self.Energy_slider.valmax:
        self.Energy_slider.set_val(self.Energy_slider.valmax)
        
    elif value<self.Energy_slider.valmin:
        self.Energy_slider.set_val(self.Energy_slider.valmin)
    
    if value<=self.Energy_slider.valmax and value>=self.Energy_slider.valmin:
       
        if hasattr(self.ax,'updateLimits'):
            self.ax.updateLimits(self.Z[0,0,value],self.Z[0,0,value+1])

        if value!=val:
            self.value = val
            self.Energy_slider.set_val(value)
            self.plot()
        else:
            self.value = val
            #self.Energy_slider.set_val(value)
            self.plot()
    if hasattr(self.ax,'_step'):
        self.ax._step=self.calculateValue()


def addColorbarSliders(self,c_min,c_max,c_minval,c_maxval,ax_cmin,ax_cmax,log=True):
    """Add two colorbars controling the colour axis

    args:

        self (Viewer3D object): Current object

        c_min (float): Minimal color value

        c_max (float): Maximal color value

        c_minval (float): Starting value of lower bound

        c_maxval (float): Starting value of upper  bound

        ax_cmin (mpl axes): Axis in which lower bound slider is to be shown
        
        ax_cmax (mpl axes): Axis in which upper bound slider is to be shown
        
    Kwargs:
    

        log (bool): If true, sliders are logarithmic

    """

    fig = self.ax.get_figure()
    if log==True:
        fig.s_cmin = Slider(ax_cmin, 'min', np.log10(c_min+1e-20), np.log10(c_max), valinit=np.log10(c_min+1e-20),orientation='vertical',valfmt='%2.1f')
        fig.s_cmax = Slider(ax_cmax, 'max', np.log10(c_min+1e-20), np.log10(c_max), valinit=np.log10(c_max),orientation='vertical',valfmt='%2.1f')
    else:
        fig.s_cmin = Slider(ax_cmin, 'min', c_min, c_max, valinit=c_min,orientation='vertical',valfmt='%2.1e')
        fig.s_cmax = Slider(ax_cmax, 'max', c_min, c_max, valinit=c_max,orientation='vertical',valfmt='%2.1e')

    fig.s_cmin._log = log
    fig.s_cmax._log = log
    for s in [fig.s_cmin,fig.s_cmax]:
        s.label.set_fontsize(12)
        s.valtext.set_fontsize(12)
    
    def update(fig,val, bar=None, s=None,log=log):
        _cmin =fig.s_cmin.val# np.log10(fig.s_cmin.val)
        _cmax =fig.s_cmax.val# np.log10(fig.s_cmax.val)
        if _cmin>_cmax:
            if bar == 'min': # If lower bar is change push max upwards
                _cmax = _cmin
                if log:
                    fig.s_cmax.set_val(np.power(10,_cmax))
                else:
                    fig.s_cmax.set_val(_cmax)
            else:
                _cmin = _cmax
                if log:
                    fig.s_cmin.set_val(np.power(10,_cmin))
                else:
                    fig.s_cmin.set_val(_cmin)
        
        if log:
            
            #self.caxis = (np.power(10,_cmin), np.power(10,_cmax))
            self.im.set_clim([np.power(10,_cmin), np.power(10,_cmax)])
            fig.s_cmax.valtext.set_text(_cmin)
            fig.s_cmax.valtext.set_text(_cmax)
            self._caxis = (np.power(10,_cmin), np.power(10,_cmax))
        else:
            
            #self.caxis = (_cmin,_cmax)
            self.im.set_clim([_cmin,_cmax])
            self._caxis = (_cmin,_cmax)
        plt.draw()
        self.colorbar.update_normal(self.im)
    
    
    fig.s_cmin.on_changed(lambda val,*arg,**kwargs: update(fig,val,*arg,bar='min',**kwargs))
    fig.s_cmax.on_changed(lambda val,*arg,**kwargs: update(fig,val,*arg,bar='max',**kwargs))
    
     
    
   
    fig._savefig = fig.savefig
    

    @functools.wraps(fig.savefig)
    def savefig(fname,hide = [ax_cmin,ax_cmax],fig=fig,**kwargs):
    
        unhide = []
        for obj in hide:
            if obj.get_visible():
                obj.set_visible(False)
                unhide.append(obj)
            
        fig._savefig(fname,**kwargs)
        for obj in unhide:
            obj.set_visible(True)
            
        fig.canvas.draw()
    
    fig.savefig = savefig

def cut1DFunctionDefault(self,dr):# pragma: no cover
    global cut1DHolder
    parameters = extractCut1DProperties(dr.rect,self.ax.sample)
    step = self.dQE[self.axis]
    pos = np.zeros(3,dtype=int)
    pos[self.axis]=self.Energy_slider.val
    
    EMin = self.bins[self.axis][pos[0],pos[1],pos[2]]
    EMax = EMin+step
    cut1DHolder.append([self.ds.plotCut1D(**parameters,Emin=EMin,Emax=EMax)])




from MJOLNIR.Geometry.InstrumentXML import createXMLString, parseXML


def test_parseXML(): # Improve this test!
	from MJOLNIR.Geometry import Wedge,Analyser,Detector,Instrument
	import os
	tempFileName = '__temp__'
		
	Instr = Instrument.Instrument()
	Instr.settings['Author'] = 'Jakob Lass'

	wedge = Wedge.Wedge(position=(0.5,0,0))

	Det = Detector.TubeDetector1D(position=(1.0,1,0),direction=(1,0,0))
	Ana = Analyser.FlatAnalyser(position=(0.5,0,0),direction=(1,0,1))

	wedge.append([Det,Ana])
	Instr.append([wedge,wedge])
	Instr.append(wedge)
		
	f = open(tempFileName,'w')

	f.write(createXMLString(Instr))
	f.close()
		
		
	InstrLoaded = parseXML(tempFileName)
	os.remove(tempFileName)
		
	assert(Instr==InstrLoaded)


def test_parseWrongXML():
	import os
	tempFileName = '__temp__'
	with open(tempFileName,'w') as f:
		f.write("<Instrument Initialized='False' Author='Jakob Lass' Date ='16/03/18' position='0.0,0.0,0.0'>n"\
+"	<WrongWedge position='0.0,0.0,0.0' concept='ManyToMany'>\n"\
+"		<FlatAnalyser position='-0.0,1.1195,0.0' direction='0.7071067811865476,-0.0,0.0' d_spacing='3.354' mosaicity='60' width='0.05' height='0.1'></FlatAnalyser>\n"\
+"		<FlatAnalyser position='-0.0,1.1827,0.0' direction='0.7071067811865475,-0.0,0.0' d_spacing='3.354' mosaicity='60' width='0.05' height='0.1'></FlatAnalyser>\n"\
+"		<TubeDetector1D position='-0.01151899615488017,1.1999447123628586,0.71' direction='-0.01151899615488017,1.1999447123628586,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"		<TubeDetector1D position='-0.0,1.2,0.7' direction='-0.0,1.2,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"		<TubeDetector1D position='0.01151899615488017,1.1999447123628586,0.71' direction='0.01151899615488017,1.1999447123628586,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"	</WrongWedge>\n"\
+"</Instrument>")

	try:
		InstrLoaded = parseXML(tempFileName)
	except ValueError: # Wrong wedge type
		assert True

	os.remove(tempFileName)

	with open(tempFileName,'w') as f:
		f.write("<Instrument Initialized='False' Author='Jakob Lass' Date ='16/03/18' position='0.0,0.0,0.0'>n"\
+"	<Wedge position='0.0,0.0,0.0' concept='ManyToMany'>\n"\
+"		<WrongFlatAnalyser position='-0.0,1.1195,0.0' direction='0.7071067811865476,-0.0,0.0' d_spacing='3.354' mosaicity='60' width='0.05' height='0.1'></WrongFlatAnalyser>\n"\
+"		<FlatAnalyser position='-0.0,1.1827,0.0' direction='0.7071067811865475,-0.0,0.0' d_spacing='3.354' mosaicity='60' width='0.05' height='0.1'></FlatAnalyser>\n"\
+"		<TubeDetector1D position='-0.01151899615488017,1.1999447123628586,0.71' direction='-0.01151899615488017,1.1999447123628586,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"		<TubeDetector1D position='-0.0,1.2,0.7' direction='-0.0,1.2,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"		<TubeDetector1D position='0.01151899615488017,1.1999447123628586,0.71' direction='0.01151899615488017,1.1999447123628586,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"	</Wedge>\n"\
+"</Instrument>")
	
	try:
		InstrLoaded = parseXML(tempFileName)
	except ValueError: # Wrong Analyser type
		assert True

	os.remove(tempFileName)

	with open(tempFileName,'w') as f:
		f.write("<Instrument Initialized='False' Author='Jakob Lass' Date ='16/03/18' position='0.0,0.0,0.0'>n"\
+"	<Wedge position='0.0,0.0,0.0' concept='ManyToMany'>\n"\
+"		<FlatAnalyser direction='0.7071067811865476,-0.0,0.0' d_spacing='3.354' mosaicity='60' width='0.05' height='0.1'></FlatAnalyser>\n"\
+"		<FlatAnalyser position='-0.0,1.1827,0.0' direction='0.7071067811865475,-0.0,0.0' d_spacing='3.354' mosaicity='60' width='0.05' height='0.1'></FlatAnalyser>\n"\
+"		<TubeDetector1D position='-0.01151899615488017,1.1999447123628586,0.71' direction='-0.01151899615488017,1.1999447123628586,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"		<TubeDetector1D position='-0.0,1.2,0.7' direction='-0.0,1.2,0.0' pixels='1024' length='1' diameter='0.02' split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"		<TubeDetector1D split='0,189, 298, 404, 510, 618, 726, 837,1024'></TubeDetector1D>\n"\
+"	</Wedge>\n"\
+"</Instrument>")
	
	try:
		InstrLoaded = parseXML(tempFileName)
	except ValueError: # No position in flat analyser
		assert True

	os.remove(tempFileName)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Wed Jul 14 13:03:12 2021

@author: Jakob Lass

Tool to replace batch in order to fully port development to windows
"""

import os,sys,shutil

DIR = os.path.join(str(os.path.dirname(__file__)),'notebooks')
RESULTDIR = os.path.join(str(os.path.dirname(__file__)),r'..\build','notebooks')

files = [f.path for f in os.scandir(DIR) if f.is_file()]

for f in files:
    if os.path.splitext(f)[1] != '.ipynb':
        continue
    sourceDate = os.path.getmtime(f)
    name = os.path.split(f)[-1]
    resultFile = os.path.join(RESULTDIR,name)
    resultDate = os.path.getmtime(resultFile)

    if sourceDate>resultDate:
        os.system('jupyter nbconvert --execute --to notebook --inplace '+f)
    else:
        print('Skipping:',name)

